# PSQF 6243 Example 4b using R

# Set width of output and  number of digits to be printed
options("width"=120, "digits"=8) 

# Use this to turn off scientific notation: 
# "fixed notation will be preferred unless it is more than 'scipen' digits wide"
options(scipen=9)

# turn off stars for different alpha levels
options(show.signif.stars=FALSE)

if (!require("haven")) install.packages("haven")
library(haven) # To import SAS data with labels as table

if (!require("expss")) install.packages("expss")
library(expss) # To add variable and value labels, sorting

if (!require("Hmisc")) install.packages("Hmisc")
library(Hmisc) # To get bivariate correlations with p-values
# Note: has routine "describe" that competes with psych package

if (!require("psych")) install.packages("psych")
library(psych) # To add descriptive summary functions

if (!require("multcomp")) install.packages("multcomp")
library(multcomp) # To do glht linear combinations in GLMs

if (!require("ppcor")) install.packages("ppcor")
library(ppcor) # To get partial correlations

if (!require("car")) install.packages("car")
library(car) # To get Type 3 sums of squares
# If interactions are included, this option is needed too
options(contrasts = c("contr.sum", "contr.poly")) 

if (!require("TeachingDemos")) install.packages("TeachingDemos")
library(TeachingDemos) # To create text output files using code below


################################################################################
#####            BEGIN EXAMPLE 4b DATA IMPORT AND MANIPULATION             #####
################################################################################

# Set working directory (to import and export files to)
# Paste in the folder address where "SAS_Chapter2.sas7bdat" is saved in quotes
setwd("C:/Dropbox/22SP_PSQF6243/PSQF6243_Example4b")

# Import GSS_Example.xlsx data from working directory -- path = file name
Example4b = read_sas(data_file="SAS_Chapter2.sas7bdat") 
# Convert to data frame to use for analysis
Example4b = as.data.frame(Example4b)

# Center quantitative predictors near their means
Example4b$age85=Example4b$age-85
Example4b$grip9=Example4b$grip-9

# Create 2 indicator-dummy-coded binary predictors for 3 dementia groups
Example4b$demNF=NA; Example4b$demNC=NA  # Create 2 new empty variables 
Example4b$demNF[which(Example4b$demgroup==1)]=0  # Replace each for none group
Example4b$demNC[which(Example4b$demgroup==1)]=0
Example4b$demNF[which(Example4b$demgroup==2)]=1  # Replace each for future group
Example4b$demNC[which(Example4b$demgroup==2)]=0
Example4b$demNF[which(Example4b$demgroup==3)]=0  # Replace each for current group
Example4b$demNC[which(Example4b$demgroup==3)]=1
# demNF: None=0 vs Future=1
# demNC: None=0 vs Current=1 

# Label variables used below (add descriptive titles) using comments instead
#   because variable labels are not allowed in many analysis packages
# Format: name: Descriptive Variable Label
#age85=     "age85: Age in Years (0=85)"
#grip9=     "grip9: Grip Strength in Pounds (0=9)"
#sexMW=     "sexMW: Sex (0=M, 1=W)"
#demNF=     "demNF: Dementia Contrast for None=0 vs Future=1"
#demNC=     "demNC: Dementia Contrast for None=0 vs Current=1"
#cognition= "cognition: Cognition Outcome"
#demgroup=  "demgroup: Dementia Group 1N 2F 3C"

# Select cases complete on all variables to be used
Example4b = Example4b[complete.cases(Example4b[ , 
            c("cognition","age","grip","sexMW","demgroup")]),]


# Open external file to save results to
txtStart(file="Example4b_R_Output.txt")


################################################################################
#####    ANALYSES: DESCRIPTION, CORRELATION, GENERAL LINEAR MODELS        #####
################################################################################

print("R Descriptive Statistics for Quantitative Variables")
describe(x=Example4b[ , c("cognition","age","grip","sexMW")])

print("R Descriptive Statistics for Categorical Variables")
prop.table(table(x=Example4b$sexMW,useNA="ifany"))
prop.table(table(x=Example4b$demgroup,useNA="ifany"))

txtComment(" ") # insert blank space in output

print("R Pearson Correlation Matrix with P-values using rcorr from Hmisc package")
print("Must convert data frame to matrix to use rcorr")
rcorr(x=as.matrix(Example4b[,c("cognition","age","grip","sexMW")]), type="pearson")

txtComment(" ") # insert blank space in output

print("R Empty Model Predicting Cognition")
ModelEmpty = lm(data=Example4b, formula=cognition~1)
anova(ModelEmpty)   # anova to print residual variance
summary(ModelEmpty) # summary to print fixed effects solution 
confint(ModelEmpty, level=.95) # confint for level% CI for fixed effects

# Save sums of squares from empty model for later calculations
SStotal=anova(ModelEmpty)$`Sum Sq`

txtComment(" ") # insert blank space in output

print("R Linear + Quadratic Age (0=85) Predicting Cognition")
ModelQAge = lm(data=Example4b, formula=cognition~1+age85+I(age85^2))
anova(ModelQAge)   # anova to print residual variance
summary(ModelQAge) # summary to print fixed effects solution 
confint(ModelQAge, level=.95) # confint for level% CI for fixed effects

txtComment(" ") # insert blank space in output

print("R Linear + Quadratic Grip (0=9) Predicting Cognition")
ModelQGrip = lm(data=Example4b, formula=cognition~1+grip9+I(grip9^2))
anova(ModelQGrip)   # anova to print residual variance
summary(ModelQGrip) # summary to print fixed effects solution 
confint(ModelQGrip, level=.95) # confint for level% CI for fixed effects

txtComment(" ") # insert blank space in output


print("R 3-Category Dementia Group (Ref=None) Predicting Cognition")
ModelDem = lm(data=Example4b, formula=cognition~1+demNF+demNC)
anova(ModelDem)   # anova to print residual variance
summary(ModelDem) # summary to print fixed effects solution 
confint(ModelDem, level=.95) # confint for level% CI for fixed effects

print("R Ask for predicted cognition per group and group differences")
print("In number lists below, values are multiplier for each fixed effect IN ORDER")
PredDem = glht(model=ModelDem, linfct=rbind(
  "Cognition Mean: None"         = c(1, 0, 0),  
  "Cognition Mean: Future"       = c(1, 1, 0), 
  "Cognition Mean: Current"      = c(1, 0, 1), 
  "Mean Diff: None vs Future"    = c(0, 1, 0),
  "Mean Diff: None vs Current"   = c(0, 0, 1),
  "Mean Diff: Future vs Current" = c(0,-1, 1)))
print("Save glht linear combination results with unadjusted p-values and 95% CIs")
SavePredDem = summary(PredDem, test=adjusted("none")) 
print(SavePredDem); confint(PredDem, level=.95, calpha=univariate_calpha())

# R code to compute effect sizes from stored GLHT results
PredDemPartialD=(2*SavePredDem$test$tstat)/sqrt(ModelDem$df.residual)
PredDemPartialR=SavePredDem$test$tstat/
             sqrt(SavePredDem$test$tstat^2+ModelDem$df.residual)
# Concatenate effect sizes to GLHT table for mean differences
data.frame(Estimate=SavePredDem$test$coefficients, SE=SavePredDem$test$sigma,
           pvalue=SavePredDem$test$pvalues, 
           PartialD=PredDemPartialD, PartialR=PredDemPartialR)

txtComment(" ") # insert blank space in output


print("R Combined Model Predicting Cognition")
ModelAll = lm(data=Example4b, formula=cognition~1+age85+grip9+sexMW+demNF+demNC)
anova(ModelAll)            # anova to print residual variance
Anova(ModelAll,Type="3")   # Anova from car package to get Type 3 SS
SumModelAll = summary(ModelAll); SumModelAll # save summary, print fixed effects solution
confint(ModelAll, level=.95) # confint for level% CI for fixed effects

print("R Ask for missing model-implied group difference")
PredModelAll = glht(model=ModelAll, linfct=rbind("Future vs Current" = c(0,0,0,0,-1,1)))
print("Save glht linear combination results with unadjusted p-values and 95% CIs")
SaveModelAll = summary(PredModelAll, test=adjusted("none")) 
print(SaveModelAll); confint(PredModelAll, level=.95, calpha=univariate_calpha())

print("Replicate F-test for Model: includes all 5 slopes") 
ModelAllFall = glht(model=ModelAll, linfct=c("age85=0","grip9=0","sexMW=0","demNF=0","demNC=0"))
summary(ModelAllFall, test=Ftest()) # ask for joint hypothesis test instead of separate 

print("Ask for F-test for overall effect of DemGroup") 
ModelAllFdem = glht(model=ModelAll, linfct=c("demNF=0","demNC=0"))
summary(ModelAllFdem, test=Ftest()) # ask for joint hypothesis test instead of separate 

print("R Semi-Partial R2 for DemGroup via SS for Reduced Model Omitting DemGroup")
ModelNoDem = lm(data=Example4b, formula=cognition~1+age85+grip9+sexMW)
SSDem=anova(ModelNoDem, ModelAll); SSDem$`Sum of Sq`/SStotal

print("Ask for F-test for effects of Age and Sex") 
ModelAllFagesex = glht(model=ModelAll, linfct=c("age85=0","sexMW=0"))
summary(ModelAllFagesex, test=Ftest()) # ask for joint hypothesis test instead of separate 

print("R Semi-Partial R2 for Age and Sex via SS for Reduced Model Omitting Age and Sex")
ModelAllNoAgeSex = lm(data=Example4b, formula=cognition~1+grip9+demNF+demNC)
SSAgeSex=anova(ModelAllNoAgeSex, ModelAll); SSAgeSex$`Sum of Sq`/SStotal

print("R code to compute effect sizes from stored model results from GLHT")
DemPartialD=(2*SaveModelAll$test$tstat)/sqrt(ModelAll$df.residual)
DemPartialR=SaveModelAll$test$tstat/sqrt(SaveModelAll$test$tstat^2+ModelAll$df.residual)
# Concatenate effect sizes to results table for mean differences
data.frame(Estimate=SaveModelAll$test$coefficients, SE=SaveModelAll$test$sigma,
           pvalue=SaveModelAll$test$pvalues, 
           PartialD=DemPartialD, PartialR=DemPartialR)

print("R code to compute effect sizes from stored model results from fixed effects")
ModelAllpD=(2*SumModelAll$coefficients[,"t value"])/sqrt(ModelAll$df.residual)
ModelAllpR=   SumModelAll$coefficients[,"t value"]/
         sqrt(SumModelAll$coefficients[,"t value"]^2+ModelAll$df.residual)
# Concatenate effect sizes to results table for fixed effects
data.frame(SumModelAll$coefficients, PartialD=ModelAllpD, PartialR=ModelAllpR)



# Close output text file
txtStop()


